import { Construct, IConstruct } from 'constructs';
import { Chart } from './chart';
import { ApiObjectMetadata, ApiObjectMetadataDefinition } from './metadata';
/**
 * Options for defining API objects.
 */
export interface ApiObjectOptions {
    /**
     * Object metadata.
     *
     * If `name` is not specified, an app-unique name will be allocated by the
     * framework based on the path of the construct within thes construct tree.
     */
    readonly metadata?: ApiObjectMetadata;
    /**
     * API version.
     */
    readonly apiVersion: string;
    /**
     * Resource kind.
     */
    readonly kind: string;
    /**
     * Additional attributes for this API object.
     */
    readonly [key: string]: any;
}
export declare class ApiObject extends Construct {
    private readonly options;
    /**
     * The name of the API object.
     *
     * If a name is specified in `metadata.name` this will be the name returned.
     * Otherwise, a name will be generated by calling
     * `Chart.of(this).generatedObjectName(this)`, which by default uses the
     * construct path to generate a DNS-compatible name for the resource.
     */
    readonly name: string;
    /**
     * The object's API version.
     */
    readonly apiVersion: string;
    /**
     * The object kind.
     */
    readonly kind: string;
    /**
     * The chart in which this object is defined.
     */
    readonly chart: Chart;
    /**
     * Metadata associated with this API object.
     */
    readonly metadata: ApiObjectMetadataDefinition;
    /**
     * Defines an API object.
     *
     * @param scope the construct scope
     * @param ns namespace
     * @param options options
     */
    constructor(scope: Construct, ns: string, options: ApiObjectOptions);
    /**
     * Create a dependency between this ApiObject and other constructs.
     * These can be other ApiObjects, Charts, or custom.
     *
     * @param dependencies the dependencies to add.
     */
    addDependency(...dependencies: IConstruct[]): void;
    /**
     * Renders the object to Kubernetes JSON.
     */
    toJson(): any;
}
